from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
               select subject_identifier,
                      v.visit_code,
                      v.visit_code_sequence,
                      cd4,
                      cd4_date,
                      crf.site_id,
                      crf.user_created,
                      crf.user_modified,
                      crf.modified
               from meta_subject_patienthistory as crf
                        left join meta_subject_subjectvisit as v on crf.subject_visit_id = v.id
               where cd4 is null
                  or cd4_date is null \
               """
    sql_view = SqlViewGenerator(
        report_model="meta_reports.patienthistorymissingbaselinecd4view",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """select off.*,
                         CAST(NULL AS CHAR(15)) AS `visit_code`,
                         CAST(NULL AS UNSIGNED) AS `visit_code_sequence`
                  from (select subject_identifier,
                               report_datetime,
                               offschedule_datetime,
                               site_id,
                               "offschedule" as source
                        from meta_prn_offschedule
                        UNION
                        select subject_identifier,
                               report_datetime,
                               offschedule_datetime,
                               site_id,
                               "offschedule_pregnancy" as source
                        from meta_prn_offschedulepregnancy
                        UNION
                        select subject_identifier,
                               report_datetime,
                               offschedule_datetime,
                               site_id,
                               "offschedule_postnatal" as source
                        from meta_prn_offschedulepostnatal
                        UNION
                        select subject_identifier,
                               report_datetime,
                               offschedule_datetime,
                               site_id,
                               "offschedule_dm_referral" as source
                        from meta_prn_offscheduledmreferral) as off
                           left join meta_prn_endofstudy as eos
                                     on off.subject_identifier = eos.subject_identifier
                  where eos.subject_identifier is null \
               """

    sql_view = SqlViewGenerator(
        report_model="meta_reports.eosreportview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """select screening_identifier,
                         site_id,
                         report_datetime          as `screening_datetime`,
                         fbg_datetime,
                         converted_fbg_value      as `fbg_value`,
                         converted_ogtt_value     as `ogtt_value`,
                         fbg2_value,
                         ogtt2_value,
                         repeat_glucose_performed as `repeated`,
                         p3_ltfu,
                         fbg2_datetime,
                         ogtt2_datetime,
                         consented,
                         screening_identifier     as `subject_identifier`,
                         id                       as `original_id`,
                         CAST(NULL AS CHAR(15))   AS `visit_code`,
                         CAST(NULL AS UNSIGNED)   AS `visit_code_sequence`
                  from meta_screening_subjectscreening
                  where converted_fbg_value is not null
                    and converted_ogtt_value is null
                    and unsuitable_agreed != "Yes" \
               """
    sql_view = SqlViewGenerator(
        report_model="meta_reports.missingscreeningogttview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 5.2.11 on 2026-02-24 19:47

from django.db import migrations

from edc_offstudy.constants import COMPLETED_FOLLOWUP
from meta_prn.models import OffStudyMedication


def update_reasons(apps, schema_editor):
    other_reasons = [
        "Patient has reached month 48",
        "end of study",
        "Participant has reached month48 follow up",
        "The patient has completed three years and does not wish to reconsent",
        "Participant has reached month 48-month", "completed 36 months follow up",
        "completed 36 months of follow up",
        "Patient has reached month 36 and has declined to re-consent",
        "Reached 36 months refused to re consent",
        "reached month 48",
        (
            "Patient reached month 36 asked time to think before she reconsent "
            "but eventually decided not to continue with the trial"
        ),
        "Completed 36 months of treatment",
        "Reached month 36 does not wish to continue with the trial",
        "reached month 36 and declined reconsenting",
        "Patient. has reach month 48",
    ]
    for other_reason in other_reasons:
        for obj in OffStudyMedication.objects.filter(reason_other=other_reason):
            obj.comment = (
                f"{obj.comment} - {obj.reason_other}" if obj.comment else obj.reason_other
            )
            obj.reason = COMPLETED_FOLLOWUP
            obj.save_base(update_fields=["comment", "reason"])


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_prn",
            "0069_alter_historicaloffstudymedication_singleton_field_and_more",
        ),
    ]

    operations = [migrations.RunPython(update_reasons)]



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
               select subject_identifier,
                      site_id,
                      appt_datetime,
                      `first_value`,
                      `second_value`,
                      `third_value`,
                      datediff(`third_date`, `first_date`) as `interval_days`,
                      datediff(now(), `first_date`)        as `from_now_days`,
                      CAST(NULL AS CHAR(15))               AS `visit_code`,
                      CAST(NULL AS UNSIGNED)               AS `visit_code_sequence`
               from (select subject_identifier,
                            site_id,
                            appt_datetime,
                            FIRST_VALUE(visit_code) OVER w as `first_value`, NTH_VALUE(visit_code, 2) OVER w as `second_value`, NTH_VALUE(visit_code, 3) OVER w as `third_value`, FIRST_VALUE(appt_datetime) OVER w as `first_date`, NTH_VALUE(appt_datetime, 3) OVER w as `third_date`
                     from edc_appointment_appointment
                     where visit_code_sequence = 0
                       and appt_status = "New"
                       and appt_datetime <= now()
                     WINDOW w as (PARTITION BY subject_identifier order by appt_datetime
                             ROWS UNBOUNDED PRECEDING)) as B
               where `second_value` is not null
                 and `third_value` is not null \
               """  # noqa
    sql_view = SqlViewGenerator(
        report_model="meta_reports.unattendedthreeinrowview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 5.2.11 on 2026-03-02 17:40

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0062_auto_20251004_0106"),
    ]

    operations = [
        migrations.CreateModel(
            name="EosReport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField(default=django.utils.timezone.now)),
                ("offschedule_datetime", models.DateTimeField(null=True)),
                ("source", models.CharField(default="", max_length=35)),
            ],
            options={
                "verbose_name": "End of study report",
                "verbose_name_plural": "End of study report",
                "db_table": "meta_reports_eosreportview",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
    ]



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin

from ...admin_site import meta_reports_admin
from ...models import EosReport


@admin.register(EosReport, site=meta_reports_admin)
class EosReportAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ("site", "subject_identifier", "offschedule_datetime")
    include_note_column = False
    list_display = (
        "dashboard",
        "subject_identifier_link",
        "site",
        "offschedule_datetime",
        "source",
    )

    list_filter = (
        "offschedule_datetime",
        "source",
    )

    search_fields = ("subject_identifier",)

    @admin.display(description="Subject Idenfifier", ordering="subject_identifier")
    def subject_identifier_link(self, obj=None):
        url = reverse("meta_reports_admin:meta_reports_eosreport_changelist")
        return render_to_string(
            "meta_reports/columns/subject_identifier_column.html",
            {
                "subject_identifier": obj.subject_identifier,
                "url": url,
                "title": "Click to filter for this subject only",
            },
        )

    def get_subject_dashboard_url_kwargs(self, obj) -> dict:
        return dict(
            subject_identifier=obj.subject_identifier,
        )



# Generated by Django 5.2.11 on 2026-03-17 01:57
import sys

from django.db import migrations
from django.db.models import Count
from tqdm import tqdm

from clinicedc_constants import NULL_STRING
from edc_qareports.models import Note
from django.apps import apps as django_apps


def update_note_models(apps, schema_editor):
    sys.stdout.write("Updating visit_code on Note model...\n")
    for report_models in (
            Note.objects.values("report_model").annotate(total=Count("id")).order_by()
    ):
        print(report_models.get("report_model"))
        model_cls = django_apps.get_model(report_models.get("report_model"))
        report_query = model_cls.objects.exclude(visit_code__isnull=True)
        for report_obj in tqdm(report_query, total=report_query.count()):
            try:
                note_obj = Note.objects.get(
                    report_model=model_cls._meta.label_lower,
                    subject_identifier=report_obj.subject_identifier,
                    visit_code=NULL_STRING,
                )
            except Note.DoesNotExist:
                pass
            else:
                note_obj.visit_code = report_obj.visit_code or NULL_STRING
                note_obj.visit_code_sequence = report_obj.visit_code_sequence or 0
                note_obj.save()

    sys.stdout.write("Done.\n")


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_qareports",
            "0023_remove_note_edc_qareports_note_report_model_subj_uniq_and_more",
         ),
        ("meta_reports", "0080_auto_20260317_0437"),
    ]

    operations = [migrations.RunPython(update_note_models)]



from django.contrib import admin
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from rangefilter.filters import DateRangeFilterBuilder

from ...admin_site import meta_reports_admin
from ...forms import HivExitReviewReportForm
from ...models import HivExitReviewReport
from ..list_filters import IsOffScheduleListFilter


@admin.register(HivExitReviewReport, site=meta_reports_admin)
class HivExitReviewReportAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ("site", "subject_identifier", "offschedule_datetime")
    include_note_column = False
    list_per_page = 10

    form = HivExitReviewReportForm

    change_list_note = (
        "View subjects without an HIV Exit Review form. "
        "Visit is the next or last visit. Hospital identifier is not searchable."
    )

    list_display = (
        "dashboard",
        "subject_identifier",
        "hospital_identifier",
        "site",
        "offschedule_date",
        "visit_code",
        "appt_date",
        "days",
        "appt_status",
        "source",
    )

    list_filter = (
        "appt_status",
        ("offschedule_datetime", DateRangeFilterBuilder()),
        ("appt_datetime", DateRangeFilterBuilder()),
        IsOffScheduleListFilter,
        "visit_code",
        "source",
    )

    search_fields = ("subject_identifier",)

    list_display_links = ("subject_identifier",)

    # @admin.display(description="Subject Identifier", ordering="subject_identifier")
    # def subject_identifier_link(self, obj=None):
    #     url = reverse("meta_reports_admin:meta_reports_hivexitreviewreport_changelist")
    #     return render_to_string(
    #         "meta_reports/columns/subject_identifier_column.html",
    #         {
    #             "subject_identifier": obj.subject_identifier,
    #             "url": url,
    #             "title": "Click to filter for this subject only",
    #         },
    #     )

    def get_subject_dashboard_url_kwargs(self, obj) -> dict:
        return dict(
            subject_identifier=obj.subject_identifier,
        )

    @admin.display(description="Appt date", ordering="appt_datetime")
    def appt_date(self, obj):
        if obj and obj.appt_datetime:
            return obj.appt_datetime.date()
        return None

    @admin.display(description="Offschedule date", ordering="offschedule_datetime")
    def offschedule_date(self, obj):
        if obj and obj.offschedule_datetime:
            return obj.offschedule_datetime.date()
        return None



from clinicedc_constants import NULL_STRING
from django.db import models
from django_db_views.db_view import DBView
from edc_appointment.choices import APPT_STATUS
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions

from .view_definition import get_view_definition


class OffscheduleReport(QaReportModelMixin, DBView):
    source = models.CharField(max_length=35, default=NULL_STRING)

    visit_schedule_name = models.CharField(max_length=150, default=NULL_STRING)

    schedule_name = models.CharField(max_length=150, default=NULL_STRING)

    onschedule_model = models.CharField(max_length=150, default=NULL_STRING)

    onschedule_datetime = models.DateTimeField(null=True)

    visit_code = models.CharField(max_length=15, default=NULL_STRING)

    visit_code_sequence = models.IntegerField(null=True)

    appt_datetime = models.DateTimeField(null=True)

    appt_status = models.CharField(
        verbose_name="Status", choices=APPT_STATUS, max_length=25, default=NULL_STRING
    )

    days = models.IntegerField(null=True)

    view_definition = get_view_definition()

    class Meta:
        managed = False
        db_table = "meta_reports_offschedulereportview"
        verbose_name = "Subjects awaiting `Off Schedule` report"
        verbose_name_plural = "Subjects awaiting `Off Schedule` report"
        default_permissions = qa_reports_permissions



import sys

from clinicedc_constants import NULL_STRING
from django.apps import apps as django_apps
from django.core.management import BaseCommand
from django.db.models import Count
from edc_qareports.models import Note
from tqdm import tqdm


class Command(BaseCommand):
    def handle(self, *args, **options):  # noqa: ARG002
        sys.stdout.write("Updating visit_code on Note model...\n")
        for report_models in (
            Note.objects.values("report_model").annotate(total=Count("id")).order_by()
        ):
            sys.stdout.write(f"{report_models.get('report_model')}\n")
            model_cls = django_apps.get_model(report_models.get("report_model"))
            report_query = model_cls.objects.exclude(visit_code__isnull=True)
            for report_obj in tqdm(report_query, total=report_query.count()):
                try:
                    note_obj = Note.objects.get(
                        report_model=model_cls._meta.label_lower,
                        subject_identifier=report_obj.subject_identifier,
                        visit_code=NULL_STRING,
                    )
                except Note.DoesNotExist:
                    pass
                else:
                    note_obj.visit_code = report_obj.visit_code or NULL_STRING
                    note_obj.visit_code_sequence = report_obj.visit_code_sequence or 0
                    note_obj.save()

        sys.stdout.write("Done.\n")



# Generated by Django 5.2.11 on 2026-03-10 19:57

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0070_auto_20260310_2257"),
    ]

    operations = [
        migrations.CreateModel(
            name="HivExitReviewReport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField(default=django.utils.timezone.now)),
                ("source", models.CharField(default="", max_length=35)),
                ("offschedule_datetime", models.DateTimeField(null=True)),
                ("hiv_exit_data", models.CharField(default="", max_length=15)),
                ("hiv_exit_datetime", models.DateTimeField(null=True)),
                ("visit_code", models.CharField(default="", max_length=15)),
                ("visit_code_sequence", models.IntegerField(null=True)),
                ("appt_datetime", models.DateTimeField(null=True)),
                (
                    "appt_status",
                    models.CharField(
                        choices=[
                            ("new", "Not started"),
                            ("in_progress", "In Progress"),
                            ("incomplete", "Incomplete"),
                            ("done", "Done"),
                            ("cancelled", "Cancelled"),
                            ("skipped", "Skipped as per protocol"),
                        ],
                        default="",
                        max_length=25,
                        verbose_name="Status",
                    ),
                ),
                ("days", models.IntegerField(null=True)),
            ],
            options={
                "verbose_name": "HIV Exit Review report",
                "verbose_name_plural": "HIV Exit Review report",
                "db_table": "meta_reports_hivexitreviewreportview",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
    ]



# Generated by Django 5.2.11 on 2026-03-16 23:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0078_auto_20260311_1533"),
    ]

    operations = [
        migrations.AddField(
            model_name="endpoints",
            name="visit_code_sequence",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="historicalmissingogttnote",
            name="visit_code",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AddField(
            model_name="historicalmissingogttnote",
            name="visit_code_sequence",
            field=models.IntegerField(blank=True, default=0, verbose_name="Sequence"),
        ),
        migrations.AddField(
            model_name="missingogttnote",
            name="visit_code",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AddField(
            model_name="missingogttnote",
            name="visit_code_sequence",
            field=models.IntegerField(blank=True, default=0, verbose_name="Sequence"),
        ),
        migrations.AlterModelTable(
            name="impsubstitutions",
            table="meta_report_impsubstitutionsview",
        ),
        migrations.AlterModelTable(
            name="missingscreeningogtt",
            table="meta_reports_missingscreeningogttview",
        ),
        migrations.AlterModelTable(
            name="onstudymissinglabvalues",
            table="meta_reports_onstudymissinglabvaluesview",
        ),
        migrations.AlterModelTable(
            name="onstudymissingvalues",
            table="meta_reports_onstudymissingvaluesview",
        ),
        migrations.AlterModelTable(
            name="patienthistorymissingbaselinecd4",
            table="meta_reports_patienthistorymissingbaselinecd4view",
        ),
        migrations.AlterModelTable(
            name="unattendedthreeinrow",
            table="meta_reports_unattendedthreeinrowview",
        ),
        migrations.AlterModelTable(
            name="unattendedthreeinrow2",
            table="meta_reports_unattendedthreeinrow2view",
        ),
        migrations.AlterModelTable(
            name="unattendedtwoinrow",
            table="meta_reports_unattendedtwoinrowview",
        ),
    ]



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
               select v.subject_identifier,
                      fbg_value,
                      fbg_units,
                      fbg_datetime,
                      null                      as `ogtt_value`,
                      null                      as `ogtt_units`,
                      null                      as `ogtt_datetime`,
                      case
                          when fasting = "fasting" then "Yes"
                          when fasting = "non_fasting" then "No"
                          else fasting end      as `fasted`,
                      fbg.site_id,
                      v.visit_code,
                      v.visit_code_sequence,
                      v.report_datetime,
                      v.appointment_id,
                      eos.offstudy_datetime,
                      fasting_duration_delta,
                      "meta_subject.glucosefbg" as source
               from meta_subject_glucosefbg as fbg
                        left join meta_subject_subjectvisit as v on v.id = fbg.subject_visit_id
                        left join meta_prn_endofstudy as eos
                                  on v.subject_identifier = eos.subject_identifier
               UNION
               select v.subject_identifier,
                      fbg_value,
                      fbg_units,
                      fbg_datetime,
                      ogtt_value,
                      ogtt_units,
                      ogtt_datetime,
                      case
                          when fasting = "fasting" then "Yes"
                          when fasting = "non_fasting" then "No"
                          else fasting end   as `fasted`,
                      fbg.site_id,
                      v.visit_code,
                      v.visit_code_sequence,
                      v.report_datetime,
                      v.appointment_id,
                      eos.offstudy_datetime,
                      fasting_duration_delta,
                      "meta_subject.glucose" as source
               from meta_subject_glucose as fbg
                        left join meta_subject_subjectvisit as v on v.id = fbg.subject_visit_id
                        left join meta_prn_endofstudy as eos
                                  on v.subject_identifier = eos.subject_identifier \
               """
    sql_view = SqlViewGenerator(
        report_model="meta_reports.glucosesummaryview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from clinicedc_constants import NULL_STRING
from django.db import models
from django_db_views.db_view import DBView
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions

from .view_definition import get_view_definition


class EosReport(QaReportModelMixin, DBView):
    offschedule_datetime = models.DateTimeField(null=True)

    source = models.CharField(max_length=35, default=NULL_STRING)

    view_definition = get_view_definition()

    class Meta:
        managed = False
        db_table = "meta_reports_eosreportview"
        verbose_name = "Subjects off schedule awaiting `End of Study` report"
        verbose_name_plural = "Subjects off schedule awaiting `End of Study` report"
        default_permissions = qa_reports_permissions



# Generated by Django 5.2.11 on 2026-02-24 19:05

from django.db import migrations

from edc_list_data import site_list_data
from meta_subject.models import HivExitReview
from meta_lists.models import ArvRegimens


def update_taf_ld(apps, schema_editor):
    site_list_data.load_data()
    arv_regimen = ArvRegimens.objects.get(name="TAF_LD")
    for obj in HivExitReview.objects.filter(other_current_arv_regimen="TAF-LD"):
        obj.current_arv_regimen = arv_regimen
        obj.other_current_arv_regimen = None
        obj.save()
    arv_regimen = ArvRegimens.objects.get(name="ABC_3TC_DTG")
    for obj in HivExitReview.objects.filter(
            other_current_arv_regimen__in=["ABC+3TC+DTG", "ABC/3TC +DTG"]
    ):
        obj.current_arv_regimen = arv_regimen
        obj.other_current_arv_regimen = None
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0242_glucose_ogtt_diagnostic_device_and_more"),
    ]

    operations = [migrations.RunPython(update_taf_ld)]



from .metadata_rules import (
    Eq53dlRuleGroup,
    GlucoseRuleGroup,
    HbA1cCrfRuleGroup,
    HbA1cRequisitionRuleGroup,
    HealthEconomicsRuleGroup,
    HealthEconomicsUpdateRuleGroup,
    HivExitReviewRuleGroup,
    InsulinCrfRuleGroup,
    InsulinRequisitionRuleGroup,
    LastVisitRuleGroup,
    MnsiTestRuleGroup,
    NextAppointmentRuleGroup,
    Sf12RuleGroup,
)

__all__ = ["HealthEconomicsRuleGroup"]



# Generated by Django 5.2.11 on 2026-03-02 18:08

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0064_auto_20260302_2044"),
    ]

    operations = [
        migrations.CreateModel(
            name="OffscheduleReport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField(default=django.utils.timezone.now)),
                ("source", models.CharField(default="", max_length=35)),
                ("visit_schedule_name", models.CharField(default="", max_length=150)),
                ("schedule_name", models.CharField(default="", max_length=150)),
                ("onschedule_model", models.CharField(default="", max_length=150)),
                ("onschedule_datetime", models.DateTimeField(null=True)),
            ],
            options={
                "verbose_name": "Subjects awaiting `Off Schedule` report",
                "verbose_name_plural": "Subjects awaiting `Off Schedule` report",
                "db_table": "meta_reports_offschedulereportview",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
        migrations.AlterModelOptions(
            name="eosreport",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "Subjects off schedule awaiting `End of Study` report",
                "verbose_name_plural": "Subjects off schedule awaiting `End of Study` report",
            },
        ),
    ]



from django.contrib.admin import SimpleListFilter

from ..constants import NOT_SUBMITTED, SUBMITTED
from ..models import EndOfStudy


class EosListFilter(SimpleListFilter):
    title = "End of Study"
    parameter_name = "eos"
    model_cls = EndOfStudy

    def lookups(self, request, model_admin):  # noqa: ARG002
        return (
            (SUBMITTED, "Submitted"),
            (NOT_SUBMITTED, "Not submitted"),
        )

    def queryset(self, request, queryset):  # noqa: ARG002
        qs = None
        if self.value():
            subject_identifiers = EndOfStudy.objects.values_list("subject_identifier").all()
            if self.value() == SUBMITTED:
                qs = queryset.filter(subject_identifier__in=subject_identifiers)
            elif self.value() == NOT_SUBMITTED:
                qs = queryset.exclude(subject_identifier__in=subject_identifiers)
        return qs



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    """List subjects with or without an hiv exit review.

    Incldue data fields on the last appointment"""
    subquery = """
               select history.subject_identifier,
                      scr.hospital_identifier,
                      history.site_id,
                      history.offschedule_datetime,
                      hiv.available                       as hiv_exit_data,
                      hiv.report_datetime                 as hiv_exit_datetime,
                      appt.visit_code,
                      appt.visit_code_sequence,
                      appt.appt_datetime,
                      appt.appt_status,
                      datediff(appt.appt_datetime, now()) as days,
                      'hivexitreview'                    as source
               from edc_visit_schedule_subjectschedulehistory as history
                        left join (select *
                                   from (select subject_identifier,
                                                visit_code,
                                                visit_code_sequence,
                                                appt_status,
                                                appt_datetime,
                                                ROW_NUMBER() OVER (
                                                    PARTITION BY subject_identifier
                                                    ORDER BY appt_datetime DESC
                                                ) as row_num
                                         from edc_appointment_appointment
                                         where appt_datetime < date ("2026-06-01")) as A
               where row_num = 1) as appt
               on history.subject_identifier = appt.subject_identifier
                   left join meta_subject_hivexitreview as hiv
                   on hiv.singleton_field = history.subject_identifier
                   left join meta_screening_subjectscreening as scr
                   on history.subject_identifier = scr.subject_identifier
               where history.schedule_name='schedule' and hiv.available is null
               order by history.subject_identifier;
               """
    sql_view = SqlViewGenerator(
        report_model="meta_reports.hivexitreviewreportview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from django import forms

from ..models import HivExitReviewReport


class HivExitReviewReportForm(forms.ModelForm):
    class Meta:
        model = HivExitReviewReport
        fields = ("appt_status",)



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    """List subjects missing an Off Schedule instance.

    Incldue data fields on the last appointment"""
    subquery = """
               select history.subject_identifier,
                      history.site_id,
                      history.visit_schedule_name,
                      history.schedule_name,
                      history.onschedule_model,
                      onschedule_model                    as source,
                      history.onschedule_datetime,
                      history.offschedule_model,
                      appt.visit_code,
                      appt.visit_code_sequence,
                      appt.appt_datetime,
                      appt.appt_status,
                      datediff(appt.appt_datetime, now()) as days
               from edc_visit_schedule_subjectschedulehistory as history
                        left join (select *
                                   from (select subject_identifier,
                                                visit_code,
                                                visit_code_sequence,
                                                appt_status,
                                                appt_datetime,
                                                ROW_NUMBER() OVER (
                                                    PARTITION BY subject_identifier
                                                    ORDER BY appt_datetime DESC
                                                ) as row_num
                                         from edc_appointment_appointment
                                         where appt_datetime < date("2026-06-01")) as A
                                   where row_num = 1) as appt
                                  on history.subject_identifier = appt.subject_identifier
               where history.offschedule_datetime is null
               order by history.subject_identifier; \
               """
    sql_view = SqlViewGenerator(
        report_model="meta_reports.offschedulereportview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 5.2.11 on 2026-02-25 13:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0021_diagnosticdevices"),
        ("meta_prn", "0070_auto_20260224_2247"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endofstudy",
            name="offstudy_reason",
            field=models.ForeignKey(
                limit_choices_to={
                    "name__in": [
                        "completed_followup",
                        "completed_followup_lt_36",
                        "completed_followup_lt_48",
                        "completed_followup_48",
                        "diabetes",
                        "delivery",
                        "pregnancy",
                        "clinical_withdrawal",
                        "dead",
                        "LTFU",
                        "toxicity",
                        "transferred",
                        "withdrawal",
                        "late_exclusion",
                        "OTHER",
                    ]
                },
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="meta_lists.offstudyreasons",
                verbose_name="Reason patient was terminated from the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="offstudy_reason",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={
                    "name__in": [
                        "completed_followup",
                        "completed_followup_lt_36",
                        "completed_followup_lt_48",
                        "completed_followup_48",
                        "diabetes",
                        "delivery",
                        "pregnancy",
                        "clinical_withdrawal",
                        "dead",
                        "LTFU",
                        "toxicity",
                        "transferred",
                        "withdrawal",
                        "late_exclusion",
                        "OTHER",
                    ]
                },
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="meta_lists.offstudyreasons",
                verbose_name="Reason patient was terminated from the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="reason",
            field=models.CharField(
                choices=[
                    ("completed_followup", "Completed study follow up"),
                    ("pregnancy", "Pregnancy"),
                    ("sae", "Participant is experiencing a serious adverse event"),
                    (
                        "clinician",
                        "Other condition that justifies discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("investigator", " Investigator decision"),
                    ("referral", "Referral to Diabetes clinic"),
                    ("patient", "Patient decision"),
                    ("OTHER", "Other reason (specify below)"),
                ],
                max_length=25,
                verbose_name="Reason for stopping study medication",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="reason",
            field=models.CharField(
                choices=[
                    ("completed_followup", "Completed study follow up"),
                    ("pregnancy", "Pregnancy"),
                    ("sae", "Participant is experiencing a serious adverse event"),
                    (
                        "clinician",
                        "Other condition that justifies discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("investigator", " Investigator decision"),
                    ("referral", "Referral to Diabetes clinic"),
                    ("patient", "Patient decision"),
                    ("OTHER", "Other reason (specify below)"),
                ],
                max_length=25,
                verbose_name="Reason for stopping study medication",
            ),
        ),
    ]



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
               select s.subject_identifier,
                      s.sid,
                      s.dispensed_sid,
                      arm_match,
                      s.report_datetime,
                      r.allocated_datetime,
                      s.site_id,
                      s.user_created,
                      s.user_modified,
                      s.modified,
                      s.id                   as original_id,
                      CAST(NULL AS CHAR(15)) AS `visit_code`,
                      CAST(NULL AS UNSIGNED) AS `visit_code_sequence`
               from meta_pharmacy_substitutions as s
                        left join meta_rando_randomizationlist as r
                                  on r.subject_identifier = s.subject_identifier
               order by s.subject_identifier \
               """
    sql_view = SqlViewGenerator(
        report_model="meta_report.impsubstitutionsview",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """select distinct subject_identifier,
                                  site_id,
                                  `first_value`,
                                  `second_value`,
                                  `third_value`,
                                  datediff(`third_date`, `first_date`) as interval_days,
                                  datediff(now(), `first_date`)        as from_now_days,
                                  `first_status`,
                                  `second_status`,
                                  `third_status`,
                                  sum(missed)                          as missed_count,
                                  CAST(NULL AS CHAR(15))               AS `visit_code`,
                                  CAST(NULL AS UNSIGNED)               AS `visit_code_sequence`
                  from (select subject_identifier,
                               site_id,
                               appt_datetime,
                               `missed`,
                               FIRST_VALUE(appt_status) OVER w as `third_status`, NTH_VALUE(appt_status, 2) OVER w as `second_status`, NTH_VALUE(appt_status, 3) OVER w as `first_status`, FIRST_VALUE(visit_code) OVER w as `third_value`, NTH_VALUE(visit_code, 2) OVER w as `second_value`, NTH_VALUE(visit_code, 3) OVER w as `first_value`, FIRST_VALUE(appt_datetime) OVER w as `third_date`, NTH_VALUE(appt_datetime, 2) OVER w as `second_date`, NTH_VALUE(appt_datetime, 3) OVER w as `first_date`
                        from appointments WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)) as B
                  where `second_value` is not null
                    and `third_value` is not null
                    and `first_status` = "New"
                    and `second_status` = "New"
                    and `third_status` = "New"
                  group by subject_identifier, site_id, `first_value`, `second_value`,
                           `third_value`,
                           datediff(`third_date`, `first_date`),
                           datediff(now(), `first_date`),
                           `first_status`, `second_status`, `third_status`
                  order by subject_identifier, site_id"""  # noqa

    with_stmt = """with appointments as (select subject_identifier, site_id, visit_code, visit_code_sequence,
appt_datetime, case when appt_timing="missed" then "New" else appt_status end as `appt_status`,
case when appt_timing="missed" then 1 else 0 end as `missed`
from edc_appointment_appointment
where visit_code_sequence=0 and appt_datetime<=now()
order by subject_identifier, appt_datetime)"""  # noqa

    sql_view = SqlViewGenerator(
        with_stmt=with_stmt,
        report_model="meta_reports.unattendedthreeinrow2view",
        ordering=["subject_identifier", "site_id"],
    )

    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }
